﻿
Partial Class SiteMapClass
    Inherits System.Web.UI.Page

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) _
      Handles Me.Load
        SiteMap_Div.InnerHtml = SiteMap("http://www.sysengineering.ru", _
          Request.PhysicalApplicationPath)
    End Sub

    Function ReadXmlFile(ByVal XMLFileName As String, ByVal TableName As String) _
      As Data.DataTable
        Dim TempTable As Data.DataTable = Nothing
        ' Проверка, что файл существует
        If IO.File.Exists(XMLFileName) Then
            Dim mDataSet As New Data.DataSet
            Dim mStreamRead As New System.IO.FileStream(XMLFileName, _
              System.IO.FileMode.Open, IO.FileAccess.Read)
            mDataSet.ReadXml(mStreamRead)
            mStreamRead.Close()
            For Each CurrentTable As Data.DataTable In mDataSet.Tables
                If CurrentTable.TableName = TableName Then
                    TempTable = New Data.DataTable(CurrentTable.TableName)
                    For Each CurrentColumn As Data.DataColumn In CurrentTable.Columns
                        Dim TempTable_Column As New Data.DataColumn(CurrentColumn.Caption, _
                          System.Type.GetType("System.String"))
                        TempTable.Columns.Add(TempTable_Column)
                    Next CurrentColumn
                    For Each CurrentRow As Data.DataRow In CurrentTable.Rows
                        Dim TempTable_Row As Data.DataRow = TempTable.NewRow
                        For Each CurrentColumn As Data.DataColumn In CurrentTable.Columns
                            TempTable_Row.Item(CurrentColumn.Caption) = _
                              CurrentRow(CurrentColumn).ToString
                        Next CurrentColumn
                        TempTable.Rows.Add(TempTable_Row)
                    Next CurrentRow
                End If
            Next CurrentTable
        End If
        Return TempTable
    End Function

    Function CountSymbols(ByVal InLine As String, ByVal Symbol As Char) As Integer
        Dim mSymbolCounter As Integer = 0
        For mCurrentCounter As Integer = 0 To Len(InLine) - 1
            If InLine(mCurrentCounter) = Symbol Then mSymbolCounter = mSymbolCounter + 1
        Next
        Return mSymbolCounter
    End Function

    Function SiteMap(ByVal SiteBaseName As String, ByVal SitePhysicalPath As String) _
      As String
        ' Таблица для данных файла sitemap.xml
        Dim mTempTable As New Data.DataTable
        ' Таблица для построения карты сайта
        Dim mSiteMapTable As New Data.DataTable("SiteMap")
        Dim mLinkColumn As New Data.DataColumn("Link")
        Dim mTitleColumn As New Data.DataColumn("Title")
        Dim mLevelColumn As New Data.DataColumn("Level")
        mSiteMapTable.Columns.Add(mLinkColumn)
        mSiteMapTable.Columns.Add(mTitleColumn)
        mSiteMapTable.Columns.Add(mLevelColumn)
        ' Заполнение таблицы данными из sitemap.xml
        mTempTable = ReadXmlFile(SitePhysicalPath & "sitemap.xml", "url")
        For Each mTempMapRow As Data.DataRow In mTempTable.Rows
            ' Фильтрация только страниц .aspx
            If Right(mTempMapRow(0), 5) = ".aspx" Then
                ' Построение карты сайта
                Dim mCurrentLink As String = Nothing
                Dim mCurrentTitle As String = Nothing
                Dim mCurrentLevel As Integer = 0
                Dim mCurrentSiteMapRow As Data.DataRow
                Dim mCurrentStreamReader As IO.StreamReader
                Dim mCurrentLine As String = Nothing
                ' Получение ссылки страницы отсечением базового имени
                mCurrentLink = Mid(mTempMapRow(0), Len(SiteBaseName) + 1, _
                  Len(mTempMapRow(0)) - Len(SiteBaseName))
                ' Проверка существования страницы
                If IO.File.Exists(SitePhysicalPath & mCurrentLink) Then
                    ' Подключение к страницы для считывания содержимого
                    mCurrentStreamReader = New IO.StreamReader(SitePhysicalPath _
                      & mCurrentLink)
                    ' Поиск в содержимом страницы тега <meta name="Description">
                    mCurrentLine = mCurrentStreamReader.ReadLine
                    Do While Not mCurrentLine Is Nothing
                        If InStr(LCase(mCurrentLine), LCase("<meta name=""Description""")) _
                          Then Exit Do
                        mCurrentLine = mCurrentStreamReader.ReadLine
                    Loop
                    If Not (mCurrentLine Is Nothing) Then
                        ' Получение из тега <meta name="Description"> имени страницы
                        Dim mTempStart As Integer = InStr(LCase(mCurrentLine), _
                          LCase("content=""")) + 9
                        Dim mTempEnd As Integer = InStr(mTempStart, LCase(mCurrentLine), _
                          LCase(""""))
                        mCurrentTitle = Mid(mCurrentLine, mTempStart, mTempEnd - mTempStart)
                        ' Отсечение имен корневых страниц разделов
                        If InStr(LCase(mCurrentLink), LCase("Default.aspx")) > 0 Then _
                          mCurrentLink = Left(mCurrentLink, InStr(LCase(mCurrentLink), _
                          LCase("Default.aspx")) - 1)
                        ' Определение вложенности страницы
                        mCurrentLevel = CountSymbols(mCurrentLink, "/")
                        If Right(LCase(mCurrentLink), 1) = "/" Then mCurrentLevel = _
                          mCurrentLevel - 1
                        ' Добавление заполненной строки к таблицы карты сайта
                        mCurrentSiteMapRow = mSiteMapTable.NewRow()
                        mCurrentSiteMapRow(0) = mCurrentLink
                        mCurrentSiteMapRow(1) = mCurrentTitle
                        mCurrentSiteMapRow(2) = mCurrentLevel
                        mSiteMapTable.Rows.Add(mCurrentSiteMapRow)
                    End If
                End If
            End If
        Next
        ' Создание HTML-структуры страницы карты сайта
        Dim mSiteMapText As New StringBuilder
        For mCurrentRow As Integer = 0 To mSiteMapTable.Rows.Count - 1
            Dim mSpaces As String = Nothing
            For mSpaceCounter As Integer = 0 To _
              CInt(mSiteMapTable.Rows(mCurrentRow).Item(2)) * 5
                mSpaces = mSpaces & "&nbsp;"
            Next
            mSiteMapText.AppendLine("<div>" _
              & mSpaces _
              & "<a href=""" & SiteBaseName & mSiteMapTable.Rows(mCurrentRow).Item(0) _
              & """>" _
              & mSiteMapTable.Rows(mCurrentRow).Item(1) & "</a>" _
              & "</div>")
        Next
        ' Возвращение карты сайта
        Return mSiteMapText.ToString
    End Function

End Class
